#include <Arduino.h>

#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define SCREEN_WIDTH 128 // OLED display width, in pixels
#define SCREEN_HEIGHT 64 // OLED display height, in pixels
#define OLED_RESET     -1 // Reset pin # (or -1 if sharing Arduino reset pin)
#define SCREEN_ADDRESS 0x3c ///< See datasheet for Address; 0x3D for 128x64, 0x3C for 128x32
Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);

#include <TinyGPS++.h> //http://arduiniana.org/libraries/tinygpsplus/
static const uint32_t GPSBaud = 9600;

TinyGPSPlus gps;
HardwareSerial Serial2(USART2);

static void smartDelay(unsigned long ms);

void setup()
{
  Serial.begin(115200);
  pinMode(LED_BUILTIN, OUTPUT);
  digitalWrite(LED_BUILTIN, HIGH);

  // SSD1306_SWITCHCAPVCC = generate display voltage from 3.3V internally
  if(!display.begin(SSD1306_SWITCHCAPVCC, SCREEN_ADDRESS)) {
    Serial.println(F("\nSSD1306 allocation failed"));
    for(;;); // Don't proceed, loop forever
  }
  Serial.println(F("\nSSD1306 allocation success"));
  display.display();
  delay(2000); // Pause for 2 seconds

  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  display.setCursor(0,0);
  display.println("GPS example");
  display.println(TinyGPSPlus::libraryVersion());
  display.display();
  delay(2000);
  Serial2.begin(GPSBaud);
}

void loop()
{
  display.clearDisplay();
  display.setCursor(0,0);
  display.print("DATE:    ");Serial.print("DATE: ");
  display.print(gps.date.year());Serial.print(gps.date.year());
  display.print("-");Serial.print("-");
  display.print(gps.date.month());Serial.print(gps.date.month());
  display.print("-");Serial.print("-");
  display.println(gps.date.day());Serial.println(gps.date.day());
  display.print("Satellites: ");
  Serial.print("Satellites: ");
  display.println(gps.satellites.value());
  Serial.println(gps.satellites.value());
  display.print("Time UTC  : ");Serial.print("Time UTC  : ");
  display.print(gps.time.hour()+2);Serial.print(gps.time.hour()+2);
  display.print(":");Serial.print(":");
  display.print(gps.time.minute());Serial.print(gps.time.minute());
  display.print(":");Serial.print(":");
  display.println(gps.time.second());Serial.println(gps.time.second());
  display.print("Speed     : ");Serial.print("Speed     : ");
  display.println(gps.speed.kmph());Serial.println(gps.speed.kmph());
  display.display();
  smartDelay(1000);

  if (millis() > 5000 && gps.charsProcessed() < 10)
    display.println(F("No GPS data received: check wiring"));
}

static void smartDelay(unsigned long ms)
// This custom version of delay() ensures that the gps object is being "fed".
{
  unsigned long start = millis();
  do
  {
    while (Serial2.available())
      gps.encode(Serial2.read());
  } while (millis() - start < ms);
}
